#ifndef __HOGWASH_CLIENT_CONTROL_H__
#define __HOGWASH_CLIENT_CONTROL_H__

#ifndef TRUE
#define TRUE	1
#endif

#ifndef FALSE
#define FALSE	0
#endif

/********************************************************
* cntl2.c
*
* Header file for the client part of the hogwash control
* channel
*
* hb_XXX function are blocking
* hn_XXX function are non-blocking
* hm_XXX are for misc functions
********************************************************/
#include "../common/control-common.h"
#include "../common/allcrypto.h"
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdio.h>

/****************************CONFIG DEFINES**********************/
#define MAX_TARGETS				16
#define MAX_JOBS				16
#define MAX_PACKETS				64
#define MAX_PACKET_PER_JOB		1
#define NUM_RETRIES				6
#define RETRY_TIME				2		/*in seconds*/
#define MAX_PACKET_SIZE			1514	/*in bytes*/


/****************************Type Defines************************/
#define JOB_TYPE_PING		1
#define JOB_TYPE_STAT		2
#define JOB_TYPE_GET_BUFF	3
#define JOB_TYPE_SET_BUFF	4
#define JOB_TYPE_COMMAND	5
#define JOB_TYPE_FILE_SYNC	6

#define BLOCKING_RESULT_WAITING	-1

/****************************Top Level Structs*******************/

struct packet_record;

typedef	int  (*PacketCallback)(struct packet_record* Packet, int Success, char* Data, int DataLen, void* User);
typedef void (*JobCallback)(int Success, void* data);

typedef struct target_record{
	char				InUse;
	struct in_addr		IP;
	char				Name[512];
	CryptoRecord*		Crypto;
}TargetRecord;

typedef struct job_record{	
	char				InUse;
	TargetRecord*		Target;	
	char				JobType;
	void*				JobVars;
	int					BlockingResult;
	JobCallback			Callback;
}JobRecord;

typedef struct packet_record{
	char				InUse;
	char				RawPacket[MAX_PACKET_SIZE];
	int					PacketSize;
	unsigned int		PacketID;
	char				RetriesLeft;
	PacketCallback		Callback;
	unsigned long		LastTime;
	void*				User;
	JobRecord*			Job;
}PacketRecord;



/*****************************Job Structs***********************/
typedef struct job_ping{
	char			Message[244];
	void*			User;
}JobPing;

typedef struct job_stat{
	unsigned int			BuffID;
	ControlPacketStatReply*	Stat;
	void*					User;
}JobStat;

typedef struct job_get_buff{
	unsigned int	BuffID;
	char*			TargetBuff;
	unsigned int	TargetBuffLen;
	char*			BlockStatus;
	void*			User;
}JobGetBuff;

typedef struct job_set_buff{
	unsigned int	BuffID;
	char*			TargetBuff;
	unsigned int	TargetBuffLen;
	char*			BlockStatus;
	void*			User;
}JobSetBuff;

typedef struct job_command{
	unsigned int				BuffID;
	unsigned int				CommandID;
	int							Flags;
	char						Data[230];
	ControlPacketCommandReply*	Reply;
	char*						BlockStatus;
	void*						User;
}JobCommand;

typedef struct job_file_sync{
	unsigned int	BuffID;
	unsigned int	BuffLen;
	char			FName[1024];
	FILE*			fp;
	char*			BlockStatus;
	void*			User;
}JobFileSync;


/*****************************Globals***************************/
JobRecord		Jobs[MAX_JOBS];
PacketRecord	Packets[MAX_PACKETS];
TargetRecord	Targets[MAX_TARGETS];

int				GlobalFD;


/*****************************Function Definitions*************/
/*general calls*/
int hm_init(char* fname);
void hm_close();
TargetRecord* hm_get_target_from_ip(struct in_addr ip);

/*nonblocking calls*/
int hn_get_stat(struct in_addr ip, unsigned int BuffID, ControlPacketStatReply* stat, JobCallback RFunc, void* User);
int hn_send_command(struct in_addr ip, unsigned int BuffID, unsigned int CommandID, int Flags, char* Data, int DataLen, ControlPacketCommandReply* reply, JobCallback RFunc, void* User);
int hn_ping(struct in_addr ip, char* Message, JobCallback RFunc, void* User);
int hn_get_buff(struct in_addr ip, unsigned int BuffID, char* TargetBuff, unsigned int TargetBuffLen, JobCallback RFunc, void* User);
int hn_set_buff(struct in_addr ip, unsigned int BuffID, char* TargetBuff, unsigned int TargetBuffLen, JobCallback RFunc, void* User);
int hn_file_sync(struct in_addr ip, unsigned int BuffID, unsigned int BuffLen, char* FName, JobCallback RFunc, void* User);
int hn_read_sock();

/*blocking calls*/
int hb_get_stat(struct in_addr ip, unsigned int BuffID, ControlPacketStatReply* stat);
int hb_send_command(struct in_addr ip, unsigned int BuffID, unsigned int CommandID, int Flags, char* Data, int DataLen, ControlPacketCommandReply* reply);
int hb_ping(struct in_addr ip, char* Message);
int hb_get_buff(struct in_addr ip, unsigned int BuffID, char* TargetBuff, unsigned int TargetBuffLen);
int hb_set_buff(struct in_addr ip, unsigned int BuffID, char* TargetBuff, unsigned int TargetBuffLen);
int hb_file_sync(struct in_addr ip, unsigned int BuffID, unsigned int BuffLen, char* FName);
#endif
